import { spawn } from 'child_process';

import { getArg } from './args.js';
import { error, log } from './log.js';

const ctx = 'getUser.js';

/**
 * Runs {@link getUser} concurrently for many users
 * @param {{ user: string, logs: string[] }[]} userDb userDb to run {@link getUser} on
 * @param {number} threadMax max number of threads to run concurrently
 * @param {string} directory the directory to save the user media folders in
 * @returns {Promise<void>} promise which resolves once all threads for every user have finished
 */
export const getMany = (userDb, threadMax, directory) => new Promise((resolve, reject) => {
  let running = 0;
  let index = 0;

  const get = () => {
    const onFinish = (currentIndex) => {
      log(ctx, `Finished ${userDb[currentIndex].user}/media`);
      --running;
      get();
    }

    while (running < threadMax && index < userDb.length) {
      ++running;
      let currentIndex = index++;

      let proc = getUser(userDb[currentIndex].user, directory);
      proc.stdout.on('data', data => {
        userDb[currentIndex].logs.push(data);
      });
      proc.stderr.on('data', _ => onFinish(currentIndex));
      proc.on('close', _ => onFinish(currentIndex));
      proc.on('error', _ => onFinish(currentIndex));
    }

    if (running === 0) {
      resolve();
    }
  }
  get();
});

/**
 * Retrieves gallery for specified user and saves to the specified parent path
 * @param {string} user the user to retrieve media from
 * @param {string} path the path to save the user's media folder in
 * @returns {ChildProcess} the process that was {@link spawn spawned}
 */
export const getUser = (user, path) => {
  const url = `https://twitter.com/${user}/media`;
  let args;
  try {
    args = getArg('args');
  } catch (err) {
    log(ctx, 'No args being provided to gallery-dl');
  }

  log(ctx, `python3 ~/.local/bin/gallery-dl -c ./config.json${!!args ? ' ' + args + ' ' : ' '}-d ${path} ${url}`);
  const proc = spawn(`python3 ~/.local/bin/gallery-dl -c ./config.json${!!args ? ' ' + args + ' ' : ' '}-d ${path} ${url}`, { shell: true });

  proc.stdout.on('data', data => {
    log(ctx, data);
  });
  proc.stderr.on('data', data => {
    error(ctx, data);
  });
  proc.on('error', err => {
    error(ctx, err);
  });
  proc.on('close', code => {
    log(ctx, `child process exited with code ${code}`);
  });

  return proc;
};
